unit pMainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, resample, Jpeg, ShellAPI, math, NkDib, ShlObj,
  FolderDialog, ComObj, ActiveX, ComCtrls, Buttons, INIFiles;

type
  TConvertThread = class(TThread)
  private
    { Private 錾 }
    procedure SaveJpeg;
  public
    OrgJpg : TJpegImage;
    TgtJpg : TJpegImage;
    Fname  : String;
    Filter : Integer;
  protected
    procedure Execute; override;
  end;

  TMangaPSPForm = class(TForm)
    FileList: TListBox;
    Label1: TLabel;
    OutputFolderButton: TButton;
    FolderDlg1: TFolderDlg;
    MainTimer: TTimer;
    Label2: TLabel;
    RemainLabel: TLabel;
    Label3: TLabel;
    CompressLevel: TTrackBar;
    Label4: TLabel;
    GroupBox1: TGroupBox;
    StartButton: TSpeedButton;
    StopButton: TSpeedButton;
    ConvertingFileLabel: TLabel;
    OutputFolder: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure OutputFolderButtonClick(Sender: TObject);
    procedure MainTimerTimer(Sender: TObject);
    procedure FileListDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure ConvThreadFinish(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private 錾 }
    ConvThread :TConvertThread;
    FilterType :Integer;
    Progressive:Boolean;
    procedure WMDropFiles(var Msg: TWMDROPFILES); message WM_DROPFILES;
    procedure AddItem(name: String);
    procedure ConvertFirstItem;
    function  GetOutputFolder: String;
    procedure UpdateRemains;
  public
    { Public 錾 }
  end;

var
  MangaPSPForm: TMangaPSPForm;

implementation

{$R *.dfm}

// ------------------------- Subroutine

function GetDesktopFolder: string;

  function GetDispName(shi: IShellFolder; pidl: PItemIDList): string;
  var DispName: TStrRet;
  begin
    shi.GetDisplayNameOf(pidl, SHGDN_FORPARSING, DispName);

    if DispName.uType = STRRET_CSTR then
      Result := DispName.cStr
    else if DispName.uType = STRRET_OFFSET then
      Result := PCHAR(LongInt(pidl) + LongInt(DispName.uOffset))
    else
      Result := WideCharToString(DispName.pOleStr);
  end;

  var Pidl: PItemIDList;
      DesktopFolderI: IShellFolder;

begin
  OleCheck(SHGetSpecialFolderLocation(0, CSIDL_DESKTOPDIRECTORY, Pidl));
  try
    OleCheck(SHGetDesktopFolder(DesktopFolderI));
    Result := GetDispName(DesktopFolderI, Pidl);
  finally
    CoTaskMemFree(Pidl);
  end;
end;


// ------------------------- ConvertThread

procedure TConvertThread.SaveJpeg;
begin
  try
    TgtJpg.SaveToFile(Fname);
  except
  end;
end;

procedure TConvertThread.Execute;
var
  w,h,x,y: Integer;
  OrgBmp,TgtBmp: TBitmap;
  TmpBmp       : TBitmap;
  RotOrg,RotTgt: TNkDib;
begin
  RotOrg := TNkDib.Create;
  RotTgt := TNkDib.Create;
  OrgBmp := TBitmap.Create;
  TgtBmp := TBitmap.Create;
  TmpBmp := TBItmap.Create;
  try
    OrgBmp.Assign(OrgJpg);
    h := 960;
    w := Floor(OrgJpg.Width * (h / OrgJpg.Height));
    if ((w mod 2)=1) then w := w + 1;
    //
    TgtBmp.Width  := w;
    TgtBmp.Height := h;
    Stretch(OrgBmp, TgtBmp, ResampleFilters[5].Filter, ResampleFilters[5].Width);
    //
    RotOrg.Assign(TgtBmp);
    RotTgt.Width  := h;
    RotTgt.Height := w;
    RotTgt.PixelFormat := RotOrg.PixelFormat;
    for y:=0 to w-1 do
    begin
      for x:=0 to h-1 do
      begin
        RotTgt.Pixels[x,y] := RotOrg.Pixels[w-y-1,x];
      end;
    end;
    TmpBmp.Assign(RotTgt);
    TgtJpg.Assign(TmpBmp);
    TgtJpg.Compress;
    Synchronize(SaveJpeg);
  finally
    TmpBmp.Free;
    TgtBmp.Free;
    OrgBmp.Free;
    RotTgt.Free;
    RotOrg.Free;
  end;
end;


// ------------------------- MainForm

function TMangaPSPForm.GetOutputFolder: String;
var
  f: String;
begin
  f := OutputFolder.Text;
  if ((not DirectoryExists(f)) or (f='')) then
  begin
    f := GetDesktopFolder;
  end;
  Result := IncludeTrailingPathDelimiter(f);
end;

procedure TMangaPSPForm.ConvThreadFinish(Sender: TObject);
begin
  ConvertingFileLabel.Caption := '';
  ConvThread := nil;
  if (StartButton.Down) then
  begin
    ConvertFirstItem;
  end;
end;

procedure TMangaPSPForm.ConvertFirstItem;
var
  name: String;
  path,spath: String;
begin
  if ((ConvThread = nil) and (FileList.Items.Count > 0)) then
  begin
    name := FileList.Items[0];
    FileList.Items.Delete(0);
    UpdateRemains;
    path := ExcludeTrailingPathDelimiter(ExtractFilePath(name));
    path := IncludeTrailingPathDelimiter(ExtractFileName(path));
    spath := path;
    path  := GetOutputFolder+path;
    //
    ConvThread := TConvertThread.Create(True);
    with ConvThread do
    begin
      FreeOnTerminate := True;
      OnTerminate     := ConvThreadFinish;
      OrgJpg          := TJpegImage.Create;
      TgtJpg          := TJpegImage.Create;
      FName           := path+ChangeFileExt(ExtractFileName(name),'.jpg');
      Filter          := FilterType;
      ConvertingFileLabel.Caption := spath+ExtractFileName(name);
      TgtJpg.CompressionQuality  := CompressLevel.Position * 10;
      TgtJpg.ProgressiveEncoding := Progressive;
    end;
    try
      ForceDirectories(path);
      ConvThread.OrgJpg.LoadFromFile(name);
      ConvThread.Resume;
    except
      ConvThread.TgtJpg.Free;
      ConvThread.OrgJpg.Free;
      ConvThread.Free;
      ConvThread := nil;
    end;
  end;
end;

procedure TMangaPSPForm.FormCreate(Sender: TObject);
var
  INI: TINIFile;
  f: String;
  i: Integer;
begin
  ConvThread  := nil;
  FilterType  := 5;
  Progressive := True;
  INI := TINIFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
  try
    f := INI.ReadString('Config','OutputDir','');
    if (DirectoryExists(f)) then OutputFolder.Text := f;
    i := INI.ReadInteger('Config','CompressionQuality',50);
    if ((i>=10) and (i<=100)) then CompressLevel.Position := i div 10;
    i := INI.ReadInteger('Config','ResampleFilter',5);
    if ((i>=0) and (i<=6)) then FilterType := i;
    Progressive := INI.ReadBool('Config','Progressive',True);
  finally
    INI.Free;
  end;
  DragAcceptFiles(Handle, True);
end;

procedure TMangaPSPForm.AddItem(name: String);
var
  at: Integer;
  sr: TSearchRec;
begin
  at := FileGetAttr(name);
  if ((at and faDirectory)<>0) then
  begin
    // Directory
    if (DirectoryExists(name)) then
    begin
      if (FindFirst(IncludeTrailingPathDelimiter(name)+'*.*', faAnyFile, sr)=0) then
      begin
        repeat
          if ((sr.Name<>'.') and (sr.Name<>'..')) then
          begin
            AddItem(IncludeTrailingPathDelimiter(name)+sr.Name);
          end;
        until FindNext(sr)<>0;
        FindClose(sr);
      end;

    end;
  end
    else
  begin
    if (FileExists(name)) then
    begin
      // File
      if ((UpperCase(ExtractFileExt(name))='.JPG') or (UpperCase(ExtractFileExt(name))='.JPEG')) then
      begin
        FileList.Items.Add(name);
        UpdateRemains;
      end;
    end;
  end;
end;

procedure TMangaPSPForm.WMDropFiles(var Msg: TWMDropFiles);
var
  FileName: array[0..MAX_PATH] of Char;
  i, Count: Integer;
begin
  Count := DragQueryFile(Msg.Drop, $FFFFFFFF, nil, 0);
  for i := 0 to Count - 1 do
  begin
    DragQueryFile(Msg.Drop, i, FileName, SizeOf(FileName));
    AddItem(FileName);
  end;
  DragFinish(Msg.Drop);
end;


procedure TMangaPSPForm.OutputFolderButtonClick(Sender: TObject);
begin
  if (FolderDlg1.Execute) then
  begin
    OutputFolder.Text := FolderDlg1.FolderName;
  end;
end;

procedure TMangaPSPForm.MainTimerTimer(Sender: TObject);
begin
  if (StartButton.Down) then
  begin
    ConvertFirstItem;
  end;
end;

procedure TMangaPSPForm.FileListDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  fname,path: String;
begin
  with (Control as TListBox) do
  begin
    Canvas.FillRect(Rect);

    fname := Items[Index];
    path  := ExcludeTrailingPathDelimiter(ExtractFilePath(fname));
    path  := IncludeTrailingPathDelimiter(ExtractFileName(path));
    path  := path+ExtractFileName(fname);
    Canvas.TextOut(Rect.Left, Rect.Top, path);
  end;
end;

procedure TMangaPSPForm.UpdateRemains;
begin
  RemainLabel.Caption := IntToStr(FileList.Items.Count)+'t@C';
end;

procedure TMangaPSPForm.FormDestroy(Sender: TObject);
var
  INI: TINIFile;
begin
  INI := TINIFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
  try
    INI.WriteString ('Config','OutputDir',OutputFolder.Text);
    INI.WriteInteger('Config','CompressionQuality',CompressLevel.Position*10);
    INI.WriteInteger('Config','ResampleFilter',FilterType);
    INI.WriteBool   ('Config','Progressive',Progressive); 
  finally
    INI.Free;
  end;
end;

end.


